#! /usr/bin/env python
"""
Whois module for wsgi.
"""
import cgi
import urlparse
from StringIO import StringIO
from urllib import quote
from beaker.middleware import SessionMiddleware
from fred_whois import whois


class Connection:
    def __init__(self, environ):
        self.remote_ip = environ['REMOTE_ADDR']

class Req:
    def __init__(self, environ):
        self.connection = Connection(environ)

def get_mod_python_req(environ):
    return Req(environ)

def get_url(environ):
    url = environ['wsgi.url_scheme']+'://'
    if environ.get('HTTP_HOST'):
        url += environ['HTTP_HOST']
    else:
        url += environ['SERVER_NAME']

        if environ['wsgi.url_scheme'] == 'https':
            if environ['SERVER_PORT'] != '443':
                url += ':' + environ['SERVER_PORT']
        else:
            if environ['SERVER_PORT'] != '80':
                url += ':' + environ['SERVER_PORT']
    url += quote(environ.get('SCRIPT_NAME', ''))
    url += quote(environ.get('PATH_INFO', ''))
    if environ.get('QUERY_STRING'):
        url += '?' + environ['QUERY_STRING']
    return url

def get_whois_req(environ):
    parsed_uri = urlparse.urlsplit(get_url(environ))
    return (
         environ.get('SERVER_NAME', ''),
         "python",
         get_url(environ),
         [
            parsed_uri.scheme,
            parsed_uri.netloc,
            parsed_uri.username,
            parsed_uri.password,
            parsed_uri.hostname,
            parsed_uri.port,
            parsed_uri.path,
            parsed_uri.query,
            parsed_uri.fragment,
         ],
         get_url(environ),
   )

def serve_captcha_new_code(environ, start_response, whoismod):
    "Generate new captcha and returns url to this."
    try:
        captcha_size = int(whoismod.session.get('captcha_size', 10))
    except ValueError:
        captcha_size = 10

    url = "%s://%s/whois/captcha.py?id=%s" % (environ['wsgi.url_scheme'],
                environ['HTTP_HOST'], whoismod.newCaptchaId(captcha_size))
    start_response(
        '200 OK',
        [('Content-Type', 'text/plain'),
         ('Content-Length', str(len(url)))]
    )
    return [url]


def serve_captcha_image(environ, start_response, whoismod):
    "Captcha image."
    params = cgi.parse_qs(environ['QUERY_STRING'])
    captcha_id = params.get('id', [''])[0]
    if captcha_id:
        factory = whoismod.getFactory()
        if factory:
            test = factory.get(captcha_id)
            if test:
                buff = StringIO()
                test.render(size=whois.CAPTCHA_SIZE).save(buff, "JPEG")
                image_body = buff.getvalue()
            else:
                image_body = whois.jpeg_captcha_invalid_user()
        else:
            image_body = whois.jpeg_captcha_no_access()
    else:
        image_body = whois.jpeg_captcha_invalid_user()

    start_response(
        '200 OK',
        [('Content-Type', 'image/jpeg'),
         ('Content-Length', str(len(image_body)))]
    )
    return [image_body]

def serve_pdf(environ, start_response, whoismod):
    "Captcha image."
    sid = whoismod.session.get('pdf_id', None)
    if not sid:
        raise Exception("No pdf_id in session")

    # 'cz' is used in whole enum website, 'cs' in corba
    pdflang = 'cs' if whoismod.lang == 'cz' else 'en'
    pdf = whoismod.admin.getPublicRequestPDF(sid, pdflang)

    start_response(
       "200 OK",
       [('Content-Type', 'application/pdf'),
        ('Content-Length', str(len(pdf))),
        ('Content-Disposition', 'attachment; filename=CZ-NIC_PasswordRequest.pdf')]
    )
    return [pdf]


def application(environ, start_response):
    "Main wsgi function."
    req = get_whois_req(environ)
    whoismod = whois.Whois(req, environ['beaker.session'])

    # request for captcha image
    if req[3][6] == whois.conf.get('captcha', 'location'):
        return serve_captcha_image(environ, start_response, whoismod)

    if req[3][6] == whois.conf.get('captcha', 'new_code'):
        return serve_captcha_new_code(environ, start_response, whoismod)

    # request goes to PDF request, we need Whois instance
    if req[3][6] == whois.conf.get('pdf', 'location'):
        return serve_pdf(environ, start_response, whoismod)

    form = cgi.FieldStorage(fp=environ['wsgi.input'], environ=environ)
    req = get_mod_python_req(environ)
    response_body, redirect_url = whoismod.render(
        whois.get_form_data(req, form),
        links={'registrars_list': '/whois/registrars'}
    )
    if redirect_url:
        start_response(
          '301 Redirect',
          [('Location',  redirect_url),]
        )
        return []

    start_response(
      '200 OK',
      [('Content-Type', 'text/html'),
       ('Content-Length', str(len(response_body)))]
    )
    return [response_body]


session_opts = {
    'session.auto': True,
    'session.type': 'cookie',
    'session.validate_key': 'asdasdasd',
}
application = SessionMiddleware(application, session_opts)
